{% extends "admin/base_site.html" %}
{% load i18n admin_urls static admin_modify %}

{% block extrahead %}{{ block.super }}
    <script src="{% url 'admin:jsi18n' %}"></script>
    {{ media }}
{% endblock %}

{% block extrastyle %}{{ block.super }}
    <link rel="stylesheet" type="text/css" href="{% static "admin/css/forms.css" %}">{% endblock %}
{% block coltype %}colM{% endblock %}

{% block bodyclass %}{{ block.super }} app-{{ opts.app_label }} model-{{ opts.model_name }} change-form{% endblock %}

{% block content %}
    <div id="content-main">
        <form action="#" method="post">

            {% csrf_token %}

            <div>
                <h2>{% trans 'Attention, you are adding an operation for the following wallets:' %}</h2>
                <ul>
                    {% for object in objects %}
                        <li>
                            <a href="{% url 'admin:django_advanced_wallet_wallet_change' object_id=object.pk %}">
                                {{ object }}
                            </a>
                        </li>
                    {% endfor %}
                </ul>

            </div>


            <div>
                {{ form.as_p }}
                <input type="hidden" name="task:new-operation" value="1">
                <input type="submit" class="default" style="float: none" value="{% trans 'Confirm' %}">
            </div>

        </form>
    </div>
{% endblock %}
